package java.lang;

import java.util.Comparator;
import java.util.AbstractCollection;
import java.util.AbstractSet;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;
import java.util.Map;
import java.util.HashMap;

final class ProcessEnvironment extends HashMap<String, String>
{
    private static final long serialVersionUID = -8017839552603542824L;
    static final int MIN_NAME_LENGTH = 1;
    private static final ProcessEnvironment.NameComparator nameComparator;
    private static final ProcessEnvironment.EntryComparator entryComparator;
    private static final ProcessEnvironment theEnvironment;
    private static final Map<String, String> theUnmodifiableEnvironment;
    private static final Map<String, String> theCaseInsensitiveEnvironment;
    
    private static String validateName(String s) {
        if (s.indexOf(61, 1) != -1 || s.indexOf(0) != -1) {
            throw new IllegalArgumentException("Invalid environment variable name: \"" + s + "\"");
        }
        return s;
    }
    
    private static String validateValue(String s) {
        if (s.indexOf(0) != -1) {
            throw new IllegalArgumentException("Invalid environment variable value: \"" + s + "\"");
        }
        return s;
    }
    
    private static String nonNullString(Object o) {
        if (o == null) {
            throw new NullPointerException();
        }
        return (String)o;
    }
    
    @Override
    public String put(String s, String s2) {
        return (String)super.put(validateName(s), validateValue(s2));
    }
    
    @Override
    public String get(Object o) {
        return (String)super.get(nonNullString(o));
    }
    
    @Override
    public boolean containsKey(Object o) {
        return super.containsKey(nonNullString(o));
    }
    
    @Override
    public boolean containsValue(Object o) {
        return super.containsValue(nonNullString(o));
    }
    
    @Override
    public String remove(Object o) {
        return (String)super.remove(nonNullString(o));
    }
    
    @Override
    public Set<String> keySet() {
        return new ProcessEnvironment.CheckedKeySet(super.keySet());
    }
    
    @Override
    public Collection<String> values() {
        return new ProcessEnvironment.CheckedValues(super.values());
    }
    
    @Override
    public Set<Map.Entry<String, String>> entrySet() {
        return new ProcessEnvironment.CheckedEntrySet(super.entrySet());
    }
    
    private ProcessEnvironment() {
    }
    
    private ProcessEnvironment(int initialCapacity) {
        super(initialCapacity);
    }
    
    static String getenv(String s) {
        return (String)ProcessEnvironment.theCaseInsensitiveEnvironment.get(s);
    }
    
    static Map<String, String> getenv() {
        return ProcessEnvironment.theUnmodifiableEnvironment;
    }
    
    static Map<String, String> environment() {
        return (Map)ProcessEnvironment.theEnvironment.clone();
    }
    
    static Map<String, String> emptyEnvironment(int n) {
        return new ProcessEnvironment(n);
    }
    
    private static native String environmentBlock();
    
    String toEnvironmentBlock() {
        ArrayList list = new ArrayList(this.entrySet());
        Collections.sort(list, ProcessEnvironment.entryComparator);
        StringBuilder sb = new StringBuilder(this.size() * 30);
        int compare = -1;
        for (Map.Entry entry : list) {
            String s = (String)entry.getKey();
            String s2 = (String)entry.getValue();
            if (compare < 0 && (compare = ProcessEnvironment.nameComparator.compare(s, "SystemRoot")) > 0) {
                addToEnvIfSet(sb, "SystemRoot");
            }
            addToEnv(sb, s, s2);
        }
        if (compare < 0) {
            addToEnvIfSet(sb, "SystemRoot");
        }
        if (sb.length() == 0) {
            sb.append('\0');
        }
        sb.append('\0');
        return sb.toString();
    }
    
    private static void addToEnvIfSet(StringBuilder sb, String s) {
        String getenv = getenv(s);
        if (getenv != null) {
            addToEnv(sb, s, getenv);
        }
    }
    
    private static void addToEnv(StringBuilder sb, String s, String s2) {
        sb.append(s).append('=').append(s2).append('\0');
    }
    
    static String toEnvironmentBlock(Map<String, String> map) {
        return (map == null) ? null : ((ProcessEnvironment)map).toEnvironmentBlock();
    }
    
    static String access$000(String s) {
        return validateValue(s);
    }
    
    static String access$200(Object o) {
        return nonNullString(o);
    }
    
    static {
        nameComparator = new ProcessEnvironment.NameComparator();
        entryComparator = new ProcessEnvironment.EntryComparator();
        theEnvironment = new ProcessEnvironment();
        theUnmodifiableEnvironment = Collections.unmodifiableMap(ProcessEnvironment.theEnvironment);
        String environmentBlock = environmentBlock();
        int index;
        int index2;
        for (int n = 0; (index = environmentBlock.indexOf(0, n)) != -1 && (index2 = environmentBlock.indexOf(61, n + 1)) != -1; n = index + 1) {
            if (index2 < index) {
                ProcessEnvironment.theEnvironment.put(environmentBlock.substring(n, index2), environmentBlock.substring(index2 + 1, index));
            }
        }
        (theCaseInsensitiveEnvironment = new TreeMap(ProcessEnvironment.nameComparator)).putAll(ProcessEnvironment.theEnvironment);
    }
    
    private static class CheckedEntry implements Map.Entry<String, String>
    {
        private final Map.Entry<String, String> e;
        
        public CheckedEntry(Map.Entry<String, String> e) {
            this.e = e;
        }
        
        @Override
        public String getKey() {
            return (String)this.e.getKey();
        }
        
        @Override
        public String getValue() {
            return (String)this.e.getValue();
        }
        
        @Override
        public String setValue(String s) {
            return (String)this.e.setValue(ProcessEnvironment.access$000(s));
        }
        
        @Override
        public String toString() {
            return this.getKey() + "=" + this.getValue();
        }
        
        @Override
        public boolean equals(Object o) {
            return this.e.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.e.hashCode();
        }
    }
    
    private static class CheckedEntrySet extends AbstractSet<Map.Entry<String, String>>
    {
        private final Set<Map.Entry<String, String>> s;
        
        public CheckedEntrySet(Set<Map.Entry<String, String>> s) {
            this.s = s;
        }
        
        @Override
        public int size() {
            return this.s.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.s.isEmpty();
        }
        
        @Override
        public void clear() {
            this.s.clear();
        }
        
        @Override
        public Iterator<Map.Entry<String, String>> iterator() {
            return new Iterator<Map.Entry<String, String>>() {
                Iterator<Map.Entry<String, String>> i = ProcessEnvironment.CheckedEntrySet.this.s.iterator();
                
                @Override
                public boolean hasNext() {
                    return this.i.hasNext();
                }
                
                @Override
                public Map.Entry<String, String> next() {
                    return new ProcessEnvironment.CheckedEntry((Map.Entry)this.i.next());
                }
                
                @Override
                public void remove() {
                    this.i.remove();
                }
            };
        }
        
        private static Map.Entry<String, String> checkedEntry(Object o) {
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            ProcessEnvironment.access$200(entry.getKey());
            ProcessEnvironment.access$200(entry.getValue());
            return entry;
        }
        
        @Override
        public boolean contains(Object o) {
            return this.s.contains(checkedEntry(o));
        }
        
        @Override
        public boolean remove(Object o) {
            return this.s.remove(checkedEntry(o));
        }
    }
    
    private static class CheckedValues extends AbstractCollection<String>
    {
        private final Collection<String> c;
        
        public CheckedValues(Collection<String> c) {
            this.c = c;
        }
        
        @Override
        public int size() {
            return this.c.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.c.isEmpty();
        }
        
        @Override
        public void clear() {
            this.c.clear();
        }
        
        @Override
        public Iterator<String> iterator() {
            return this.c.iterator();
        }
        
        @Override
        public boolean contains(Object o) {
            return this.c.contains(ProcessEnvironment.access$200(o));
        }
        
        @Override
        public boolean remove(Object o) {
            return this.c.remove(ProcessEnvironment.access$200(o));
        }
    }
    
    private static class CheckedKeySet extends AbstractSet<String>
    {
        private final Set<String> s;
        
        public CheckedKeySet(Set<String> s) {
            this.s = s;
        }
        
        @Override
        public int size() {
            return this.s.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.s.isEmpty();
        }
        
        @Override
        public void clear() {
            this.s.clear();
        }
        
        @Override
        public Iterator<String> iterator() {
            return this.s.iterator();
        }
        
        @Override
        public boolean contains(Object o) {
            return this.s.contains(ProcessEnvironment.access$200(o));
        }
        
        @Override
        public boolean remove(Object o) {
            return this.s.remove(ProcessEnvironment.access$200(o));
        }
    }
    
    private static final class NameComparator implements Comparator<String>
    {
        @Override
        public int compare(String s, String s2) {
            int length = s.length();
            int length2 = s2.length();
            for (int min = Math.min(length, length2), i = 0; i < min; ++i) {
                int char1 = s.charAt(i);
                int char2 = s2.charAt(i);
                if (char1 != char2) {
                    int upperCase = Character.toUpperCase((char)char1);
                    int upperCase2 = Character.toUpperCase((char)char2);
                    if (upperCase != upperCase2) {
                        return upperCase - upperCase2;
                    }
                }
            }
            return length - length2;
        }
    }
    
    private static final class EntryComparator implements Comparator<Map.Entry<String, String>>
    {
        @Override
        public int compare(Map.Entry<String, String> entry, Map.Entry<String, String> entry2) {
            return ProcessEnvironment.nameComparator.compare((String)entry.getKey(), (String)entry2.getKey());
        }
    }
}
